#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <winbase.h>
#include <atlmem.h>
#include <ATLComMem.h>
#include "C:\\Program Files (x86)\\Microsoft Visual Studio\\2019\\Community\\DIA SDK\\include\\dia2.h"
#pragma comment( lib, "C:\\Program Files (x86)\\Microsoft Visual Studio\\2019\\Community\\DIA SDK\\lib\\amd64\\diaguids.lib" )



int main( int argc, char *argv[] )
{
    if( argc != 3  ) { printf( "use: stackParser.exe input.txt input.pdb\n" ); return -1; }

    FILE *fp = fopen( argv[1], "rt" );
    if( !fp ) { printf( "Error: can't read input\n" ); return -2; }

    HRESULT hr = CoInitialize(NULL);

    CComPtr<IDiaDataSource> pSource;
    hr = CoCreateInstance( CLSID_DiaSource, NULL, CLSCTX_INPROC_SERVER, __uuidof(IDiaDataSource), (void **) &pSource);
    if (FAILED(hr))
    {
        printf( "Error: can't start .PDB reader\n" );
        return -3;
    }

    wchar_t wszFilename[ _MAX_PATH ];
    mbstowcs( wszFilename, argv[2], sizeof( wszFilename )/sizeof( wszFilename[0] ) );
    if ( FAILED( pSource->loadDataFromPdb( wszFilename ) ) )
    {
        printf( "Error: can't read .PDB file \"%s\"\n", argv[2] );
        return -4;
    }


    CComPtr<IDiaSession> psession;
    if ( FAILED( pSource->openSession( &psession ) ) )
    {
        printf( "Error: can't open .PDB file \"%s\"\n", argv[2] );
        return -5;
    }

    //===============

    const ULONGLONG add = 0x140000000;
    psession->put_loadAddress(add);

    //===============


    char str[2048];
    while( fgets( str, 2048, fp ) )
    {
        bool     valid = true;
        uint64_t add = 0;
        for( int i=0; i<16; i++ )
        {
            char c = str[i];
                 if(c>='0' && c<='9') add = (add<<4) + static_cast<uint64_t>(c-'0');
            else if(c>='a' && c<='f') add = (add<<4) + uint64_t(10+c-'a');
            else if(c>='A' && c<='F') add = (add<<4) + uint64_t(10+c-'A');
            else { valid = false; break; }
        }
        

        if( valid )
        {
            // symbol
            IDiaSymbol* ppSymbol = nullptr;
            if (FAILED(psession->findSymbolByVA(add, SymTagFunction, &ppSymbol)))
            {
                printf( "%016I64x : Failed to lookup\n", add );
                continue;
            }
            if( ppSymbol==nullptr )
            {
                printf( "%016I64x : Failed to lookup\n", add );
                continue;
            }


            BSTR functionName;
            ppSymbol->get_name(&functionName);

            // line
            DWORD lineNum = 0;
            ULONGLONG            length;
            ppSymbol->get_length( &length );
            IDiaEnumLineNumbers* pLineEnum = NULL;
            if (FAILED(psession->findLinesByVA(add, length, &pLineEnum ) ) )
            {
                lineNum = -1;
            }
            else
            {
            IDiaLineNumber *pLine = nullptr;
            ULONG celt =0;
            pLineEnum->Item(0,&pLine);
            pLine->get_lineNumber(&lineNum);
            pLine->Release();
            }
            pLineEnum->Release();

            
            wprintf( L"%016I64x : %s, line %d\n", add, functionName, lineNum );

            SysFreeString(functionName);

            ppSymbol->Release();
        }
    }

    psession.Release();
    pSource.Release();

    fclose(fp);

    CoUninitialize();

    return 0;
}